package fr.shortcircuit.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

import org.apache.commons.beanutils.BeanPredicate;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.collections.Predicate;
import org.apache.commons.collections.functors.AllPredicate;
import org.apache.commons.collections.functors.AnyPredicate;
import org.apache.commons.collections.functors.EqualPredicate;
import org.apache.commons.lang.StringUtils;

import fr.shortcircuit.model.ProductElement;
import fr.shortcircuit.util.PositiveIndexOfPredicate;

/** 
 * @author Dimitri Dean DARSEYNE (D3), 
 * Published by Short-Circuit under Creative Commons (CC) Licensing: 
 * Authorship/Paternity, NO Commercial Use, NO Derivative
 * Please check for more informations:
 * http://creativecommons.org/licenses/by-nc-nd/2.0/
 *
 * Auteur Dimitri Dean DARSEYNE (D3),
 * Publié par Short-Circuit sous license Creative Commons (CC):
 * Paternité, PAS d'Utilisation Commerciale, pas de Dérivés/Modifications
 * Pour plus d'informations, se rendre sur:
 * http://creativecommons.org/licenses/by-nc-nd/2.0/fr/ 
 * 
 * @since Short-Circuit 1999
 */

public class SearchUI implements ActionListener, IXMLViewConstants
{
	//Map content
	private Map<String, Collection<ProductElement>> 	mapType;
	private Map<String, Collection<ProductElement>> 	mapGenre;
	
	//Collection content
	private Collection<ProductElement>					collectionElt;
	
	//UI components
	private JDesktopPane 								desk;
	private JComboBox									comboPrice, comboGenre;
	private JTextField									txfFullText;
	private JTextField									txfIndexOfText;
	private JButton										butQuery;
	private JCheckBox									chkAnd, chkOr;
	private JTable										tableResults;
	private DefaultTableModel 							tableModelResults;
	private JInternalFrame 								jifTree;
	private JPanel										panelTop;
	//
	private TreeContent 								treeEltPrice;
	private TreeContent 								treeEltGenre;
	

	
	public SearchUI(JDesktopPane desk, Collection<ProductElement> collectionElt, 
					Map<String, Collection<ProductElement>> mapType, Map<String, Collection<ProductElement>> mapGenre)
	{
		setMapType(mapType);
		setMapGenre(mapGenre);
		setCollectionElt(collectionElt);
		setDesk(desk);

		buildUI();
	}

 	public void buildUI()
	{	
 		buildTreeUI();
 		buildCriteriaUI();
 		buildFrameUI();	
	}

 	public void buildTreeUI()
	{	
		treeEltPrice	= new TreeContent(mapType);
		treeEltGenre	= new TreeContent(mapGenre);
		
		treeEltPrice.setPreferredSize(new Dimension(200, 250));
		treeEltGenre.setPreferredSize(new Dimension(200, 250));
	}
 	
 	public void buildCriteriaUI()
 	{
 		//Generics use via toArray
 		String 		arrayPrice[]	= mapType.keySet().toArray(new String[]{});
 		String 		arrayGenre[]	= mapGenre.keySet().toArray(new String[]{}); 	
		ArrayList 	listPrice 		= new ArrayList();
		ArrayList 	listGenre 		= new ArrayList();

		//in order to add an empty string selection in JComboBox
 		listPrice.add("");
 		listGenre.add("");

 		listPrice.addAll(Arrays.asList(arrayPrice));
 		listGenre.addAll(Arrays.asList(arrayGenre));

 		this.panelTop				= new JPanel(); 		
 		this.comboPrice				= new JComboBox(listPrice.toArray());
 		this.comboGenre				= new JComboBox(listGenre.toArray()); 		
 		this.txfFullText			= new JTextField();
 		this.txfIndexOfText			= new JTextField();
 		this.chkAnd					= new JCheckBox("&");
 		this.chkOr					= new JCheckBox("|");
 		ButtonGroup	butgroup		= new ButtonGroup();
 		butQuery					= new JButton("Lancer");

		//Panel Container
 		panelTop.setLayout(new GridLayout(1, 11));
		panelTop.setBackground(Color.gray);
		
		panelTop.add(new JLabel("Prix"));
		panelTop.add(comboPrice);
		panelTop.add(new JLabel("Genre"));
		panelTop.add(comboGenre);
		panelTop.add(new JLabel("Text Equals"));
		panelTop.add(txfFullText);
		panelTop.add(new JLabel("Text IndexOf"));
		panelTop.add(txfIndexOfText);	
		panelTop.add(new JLabel("Search"));
		panelTop.add(chkAnd);
		panelTop.add(chkOr);
		panelTop.add(butQuery);
		panelTop.setPreferredSize(new Dimension(200, 25));
		panelTop.setVisible(true);		
		
		//Checkbox & Group
		butgroup.add(chkAnd);
		butgroup.add(chkOr);
		chkOr.setSelected(true);

		//Button & event controler
		butQuery.addActionListener(this);
 	}
 	
 	public void buildFrameUI()
 	{
 		//Table
 		this.tableModelResults 					= new DefaultTableModel(new Object[][] {}, ARRAY_COLUMNS_PROPERTY); 		
 		this.tableResults 						= new JTable(tableModelResults);
 		JScrollPane			scpTable 			= new JScrollPane(tableResults);
 		//Splitter
 		JSplitPane			splitCenter			= new JSplitPane(JSplitPane.VERTICAL_SPLIT, treeEltPrice, treeEltGenre);
 		
		splitCenter.setDividerLocation(0.50);
		
		//JInternalFrame																	 //resize, close, max, icone
		this.jifTree							= new JInternalFrame("Recherche de produits", true, true, true, true);

		jifTree.getContentPane().add(scpTable, 		BorderLayout.CENTER);
		jifTree.getContentPane().add(splitCenter, 	BorderLayout.WEST);
		jifTree.getContentPane().add(panelTop, 		BorderLayout.NORTH);
		
		desk.add(jifTree, 1);
		splitCenter.doLayout();		
		splitCenter.updateUI();
	
		jifTree.setVisible(true);
		jifTree.pack();
	
		jifTree.setBounds(2, 2, 1090, 700);								
		
		jifTree.getContentPane().doLayout();		
		jifTree.updateUI();		
	}

 	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//Action Handling
 	/////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public void actionPerformed(ActionEvent e)  	{if (e.getSource() == butQuery) search();}

	//L'interface Predicate permet d'effectuer des operations logiques atomiques sur des objects javaBeans.
	//
	//Liste des predicate disponible:
	//InstanceofPredicate, NonePredicate, NotNullPredicate, NotPredicate, NullIsExceptionPredicate, NullIsFalsePredicate, 
	//NullIsTruePredicate, NullPredicate, OnePredicate, OrPredicate, TransformedPredicate, TransformerPredicate, TruePredicate, 
	//UniquePredicate
	//
	public void search()
	{
		String strPriceSelected 	= comboPrice.getSelectedItem().toString();
		String strGenreSelected 	= comboGenre.getSelectedItem().toString();
		String strFullTextSearch	= txfFullText.getText(); 
		String strIndexOfTextSearch	= txfIndexOfText.getText();
		
		//System.out.println("SearchUI: search: strPriceSelected:" + strPriceSelected + ", strGenreSelected:" + strGenreSelected + ", strFullTextSearch:" + strFullTextSearch + ", strIndexOfTextSearch:" + strIndexOfTextSearch);
		//System.out.println("SearchUI: search: collectionElt.size:" + collectionElt.size());
			
		 try
	     {
			 ArrayList<Predicate> 		listPredicate  	= new ArrayList();
	         ArrayList<ProductElement> 	listMatching 	= new ArrayList();
	            
	         //Apache Jakarta Common StringUtils use
	         if (!StringUtils.isEmpty(strPriceSelected)) 		listPredicate.add(new BeanPredicate("price", 		new EqualPredicate(strPriceSelected)));
	         if (!StringUtils.isEmpty(strGenreSelected)) 		listPredicate.add(new BeanPredicate("genre",		new EqualPredicate(strGenreSelected)));
	         if (!StringUtils.isEmpty(strFullTextSearch))		listPredicate.add(new BeanPredicate("designation", 	new EqualPredicate(strFullTextSearch)));
	         //Specific additional PositiveIndexOfPredicate
	         if (!StringUtils.isEmpty(strIndexOfTextSearch))	listPredicate.add(new BeanPredicate("designation", 	new PositiveIndexOfPredicate(strIndexOfTextSearch)));
	            
	         //Composite Predicate Tree
	         Predicate pAll                      = (chkAnd.isSelected())? 
	            									new AllPredicate((Predicate[]) listPredicate.toArray(new Predicate[] {})) : 
	            									new AnyPredicate((Predicate[]) listPredicate.toArray(new Predicate[] {}));
	            
	        for (ProductElement p : collectionElt)
	        	if (pAll.evaluate(p))
	        		listMatching.add(p);

			//System.out.println("SearchUI: search: listMatching.size:" + listMatching.size());
	        
	        String arrayResults[][] 			= new String[listMatching.size()][ARRAY_COLUMNS_PROPERTY.length];	        
	        int indexRow						= 0;
	        
	        for (ProductElement p : listMatching)
	        {
	        	//Lecture des proprietes par la classe BeanUtils
	        	for (int indexProperty = 0; indexProperty != ARRAY_COLUMNS_PROPERTY.length; indexProperty++)
	        		arrayResults[indexRow][indexProperty]	= BeanUtils.getProperty(p, ARRAY_COLUMNS_PROPERTY[indexProperty]);	        	
	        
	        	indexRow++;
	        }
	        
	        tableModelResults 					= new DefaultTableModel(arrayResults, ARRAY_COLUMNS_PROPERTY); 		
	        
	        tableResults.setModel(tableModelResults);
	        tableResults.updateUI();
	        
	        jifTree.updateUI();
	     }
	     catch (Exception e) {System.out.println("SearchUI: search: " + e.toString()); e.printStackTrace();}          
	}
	 	
 	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//Setters
 	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	public void setMapType(Map<String, Collection<ProductElement>> mapType) 	{this.mapType 		= mapType;}
	public void setMapGenre(Map<String, Collection<ProductElement>> mapGenre) 	{this.mapGenre 		= mapGenre;}
	public void setCollectionElt(Collection<ProductElement> collectionElt) 		{this.collectionElt = collectionElt;}
	public void setDesk(JDesktopPane desk) 										{this.desk 			= desk;}

}
